/*
ch_p('.'): returns a TKn
anychar_p: returns a AnyP
nothing_p: returns a CPn
epsilon_p: returns a CPy
digit_p:   returns a digit_p
*/


struct TKn: public parser<TKn>
{
    typedef parser<TKn> base_t;
    int val;
public:
    TKn(int v) : base_t(), val(v) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
#ifdef PARSER_DEBUG
        const std::string expname = GetTokenName((QBtoken)val);
        dprintf(dp_rulematch, "Is it %s?\n", expname.c_str());
#endif
      
        if(scan.eof())
        {
#ifdef PARSER_DEBUG
            dprintf(dp_rulematch, " - Nope, end of file\n");
#endif
            return false;
        }
        typename ScannerT::iterator it = scan.pos();
        typename ScannerT::value_type got = scan.get();
        if(got == val)
        {
#ifdef PARSER_DEBUG
            dprintf(dp_rulematch, " - Yep, it was %s  (at %s)\n",
                expname.c_str(), it.GetLoc().stringify().c_str());
#endif
            return true;
        }
#ifdef PARSER_DEBUG
        dprintf(dp_rulematch, " - Nope, it was %s  (at %s)\n",
            GetTokenName((QBtoken)got).c_str(),
            it.GetLoc().stringify().c_str());
#endif
        scan.revert(it);
        return false;
    }
    
    const std::string what() const
    {
        return GetTokenName((QBtoken)val);
    }
};

template<typename A>
inline const TKn ch_p(A val) { return TKn(val); }

class AnyP: public parser<AnyP>
{
    typedef AnyP self_t;
    typedef parser<self_t> base_t;
public:
    AnyP() : base_t() { }

    template<typename ScannerT>
    inline static bool parse(ScannerT& scan)
    {
        if(scan.eof()) return false;
        scan.get();
        return true;
    }

    static const std::string what() { return "anything"; }
};
static const AnyP anychar_p = AnyP();


/* Constant parser (always the same reply without consulting the input) */
class CPy: public parser<CPy>
{
    typedef CPy self_t;
    typedef parser<self_t> base_t;
public:
    CPy() : base_t() { }

    template<typename ScannerT>
    inline static bool parse(ScannerT& /*scan*/) { return true; }

    static const std::string what() { return "success"; }
};
class CPn: public parser<CPn>
{
    typedef CPn self_t;
    typedef parser<self_t> base_t;
public:
    CPn() : base_t() { }

    template<typename ScannerT>
    inline static bool parse(ScannerT& /*scan*/) { return false; }

    static const std::string what() { return "failure"; }
};
static CPy eps_p, epsilon_p;
static CPn nothing_p;

static const struct digit_p: public parser<digit_p>
{
    typedef parser<digit_p> base_t;
public:
    digit_p() : base_t() { }
  
    template<typename ScannerT>
    static bool parse(ScannerT& scan)
    {
        if(scan.eof()) return false;
        typename ScannerT::iterator it = scan.pos();
        typename ScannerT::value_type v = scan.get();
        if(v >= '0' && v <= '9') return true;
        scan.revert(it);
        return false;
    }

    static const std::string what() { return "digit"; }
} digit_p;
